import numpy as np
#Remember to change the following for the number of files you have and the type
# of file that it is!

twoid = True


#get multiple files loaded based on number of snapshots available
#any function that takes a 't' value basically takes the abridged number of the snapshot you want.
def get_files(num):
    farray = np.zeros(num,dtype='object')
    for i in xrange(0,num):
        if i < 10:
            farray[i] = '../snapshot_00' + str(i)
        elif i < 100:
            farray[i] = '../snapshot_0'+ str(i)
        else:
            farray[i] = '../snapshot_'+ str(i)
    return farray


def get_file(t):
    if t < 10:
        filename = '../snapshot_00' + str(t)
    elif t < 100:
        filename = '../snapshot_0' + str(t)
    else:
        filename = '../snapshot_'+ str(t)
    return filename

#defines the number of particles with variable mass
def Nmass(massarr,npart):
    Nm = 0
    for j in xrange(6):
        if massarr[j] == 0:
            Nm += npart[j]
    return Nm

# unpacks the binary data from the unformattted f77 output and returns arrays
# of certain properties
def get_bindat(filename, idprompt):
    f = open(filename,'rb')  #<--rb = read binary
    dummy = np.fromfile(f,dtype='i4',count=1) # C padding 4 bytes
    npart= np.fromfile(f,dtype='i4',count=6)
    massarr = np.fromfile(f,dtype='f4',count=6)
    dummy = np.fromfile(f,dtype='f8', count=1) #time
    dummy = np.fromfile(f, dtype='f8', count=1) #redshift
    dummy = np.fromfile(f, dtype='i4',count=2) #flags
    dummy = np.fromfile(f,dtype='i4',count=6) #Nall
    N = np.sum(npart)# total number of particles
    f.seek(268) #header ends at byte 256, rest is padding
    Pos = np.fromfile(f,dtype='f4',count=3*N)
    dummy = np.fromfile(f,dtype='i4', count=2) # C padding 8 bytes
    Vel = np.fromfile(f,dtype='f4', count=3*N)
    dummy = np.fromfile(f,dtype='i4',count=2)# C padding 8 bytes
    ID = np.fromfile(f, dtype='i4',count=N)
    dummy = np.fromfile(f,dtype = 'i4', count=2)
    if idprompt == True:
        ID2 = np.fromfile(f,dtype='i4', count =N)
        dummy = np.fromfile(f,dtype='i4', count =2)
    dummy = int(Nmass(massarr,npart)) #Nm

    Vmass = np.fromfile(f, dtype='f4',count = dummy)
    Ugas = np.fromfile(f, dtype='f4',count = dummy)
    Rho = np.fromfile(f, dtype='f4',count = dummy)

    f.close()

    data = np.hstack((Pos,Vel))

    if idprompt == True:
        return N, npart,ID, ID2,data,Vmass,Rho
    if idprompt == False:
        return N, npart, ID,data,Vmass,Rho


# get sink positions at time t, in the x (0), y (1) or z (2) dimension
def xs(t,dim):
    if twoid == True:
        N, npart, ID, ID2, data, mass,rho = get_bindat(get_file(t),twoid)
    else:
        N, npart, ID, data, mass, rho = get_bindat(get_file(t), twoid)
    pos = data[0:3*N]
    x = np.reshape(pos,(N,3))
    xstar = np.ravel(x[npart[0]:,dim])
    return  xstar

# get gas positions at time t, in the x (0), y (1) or z (2) dimension
def xg(t,dim):
    if twoid == True:
        N, npart, ID, ID2, data, mass,rho = get_bindat(get_file(t),twoid)
    else:
        N, npart, ID, data, mass,rho = get_bindat(get_file(t), twoid)
    pos = data[0:3*N]
    x = np.reshape(pos,(N,3))
    index = np.where(ID > 0)
    x_a = x[index]
    xgas = np.ravel(x_a[:,dim])
    return xgas

# get gas velocities at time t, in the x (0), y (1) or z (2) dimension
def vg(t,dim):
    if twoid == True:
        N, npart, ID, ID2, data, mass, rho = get_bindat(get_file(t),twoid)
    else:
        N, npart, ID, data, mass,rho = get_bindat(get_file(t), twoid)
    vel = data[3*N:]
    v = np.reshape(vel,(N,3))
    index = np.where(ID > 0)
    v_a = v[index]
    vgas = np.ravel(v_a[:,dim])
    return vgas

# get sink velocities at time t, in the x (0), y (1) or z (2) dimension
def vs(t,dim):
    if twoid == True:
        N, npart, ID, ID2, data, mass,rho = get_bindat(get_file(t),twoid)
    else:
        N, npart, ID, data, mass,rho = get_bindat(get_file(t), twoid)
    vel = data[3*N:]
    v = np.reshape(vel,(N,3))
    vstar = np.ravel(v[npart[0]:,dim])
    return vstar

# get id arrays for all particles at time t
def ids(t):
    if twoid == True:
        N, npart, ID, ID2, data, mass,rho = get_bindat(get_file(t),twoid)
        id1 = np.ravel(ID)
        id2 = np.ravel(ID2)
        ids = np.column_stack((id1,id2))
        #return ids
        return id1[npart[0]:] #Modf 20/02/18
    else:
        N, npart, ID, data, mass,rho = get_bindat(get_file(t), twoid)
        id1= np.ravel(ID)
        return id1

# get mass of sinks at time t
def mass(t):
    #ID es para las sph (20/04/31)
    if twoid == True:
        N, npart, ID, ID2, data, mass,rho = get_bindat(get_file(t),twoid)

    else:
        N, npart, ID, data, mass,rho = get_bindat(get_file(t), twoid)

    mass_sink = np.ravel(mass[npart[0]:])
    donde = np.where(ID >= 0) #Las ID positivas corresponden a las sph que sobrevivieron
    mass_sph = np.ravel(mass[donde])

    return mass_sink,mass_sph #Agregado 20/04/31


# get density of sinks at time t
def rho_sph(t):

    if twoid == True:
        N, npart, ID, ID2, data, mass, rho = get_bindat(get_file(t),twoid)

    else:
        N, npart, ID, data, mass, rho = get_bindat(get_file(t), twoid)

    rho_a = np.ravel(rho[npart[0]:])

    return rho_a #Agregado 20/10/17
